﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace Hexagonal_Minesweeper_2019
{
    /// <summary>
    /// Interaction logic for GameOverDlgWindow.xaml
    /// </summary>
    public partial class GameOverDlgWindow : Window
    {
        public bool exitWasPressed = false;
        public bool playAgainWasPressed = false;

        public GameOverDlgWindow()
        {
            InitializeComponent();
        }

        public void SetGameData(GameData data)
        {
            if (data.gamePlayStatus == GameData.GamePlayStatus.WON)
                labelWonLostMessage.Content = "Congratulations, you won the game!";
            else
                labelWonLostMessage.Content = "Sorry, you lost this game. Better luck next time!";
            labelYouHaveFastestTime.Visibility = data.isNewBest ? Visibility.Visible : Visibility.Hidden;
            labelTime.Content = data.ElapsedTime;
            if (data.gamePlayStatus == GameData.GamePlayStatus.WON) // only show current date if player won
                labelDate.Content = DateTime.Now.ToShortDateString();
            else
            {
                labelDateLabel.Visibility = Visibility.Hidden;
                labelDate.Visibility = Visibility.Hidden;
            }
            StatisticsData.StatisicsForOneDifficulty sfod = data.Statistics.FindByDifficulty
                (data.NumMines, data.HexesWide, data.HexesHigh, data.Options.rightToLeft);
            if (sfod == null)   // should never happen!
                return;
            labelGamesPlayed.Content = sfod.gamesPlayed;
            labelGamesWon.Content = sfod.gamesWon;
            labelPercentage.Content = sfod.WinPercentage + "%";
            if (sfod.bestTimes[0] >= 999)
            {
                labelBestTimeLabel.Visibility = Visibility.Hidden;
                labelBestTime.Visibility = Visibility.Hidden;
                labelBestTimeDateLabel.Visibility = Visibility.Hidden;
                labelBestTimeDate.Visibility = Visibility.Hidden;
            }
            else
            {
                labelBestTime.Content = sfod.bestTimes[0];
                if (data.gamePlayStatus == GameData.GamePlayStatus.WON) // Only show date of best time if player lost
                {
                    labelBestTimeDateLabel.Visibility = Visibility.Hidden;
                    labelBestTimeDate.Visibility = Visibility.Hidden;
                }
                else
                    labelBestTimeDate.Content = sfod.dateOfBestTimes[0].ToShortDateString();
            }
        }

        private void buttonExit_Click(object sender, RoutedEventArgs e)
        {
            exitWasPressed = true;
            Close();
        }

        private void buttonPlayAgain_Click(object sender, RoutedEventArgs e)
        {
            playAgainWasPressed = true;
            Close();
        }
    }
}
