﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Interop;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace Hexagonal_Minesweeper_2019
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private GameData m_gameData;
        private DispatcherTimer m_timeUpdateTimer = null;
        public static RoutedCommand F1pressed = new RoutedCommand();
        public static RoutedCommand F2pressed = new RoutedCommand();
        public static RoutedCommand F4pressed = new RoutedCommand();
        public static RoutedCommand F5pressed = new RoutedCommand();

        public MainWindow()
        {
            InitializeComponent();

            System.Drawing.Bitmap bmp = Properties.Resources.TIME_ICON;
            BitmapSource bmpSource = Imaging.CreateBitmapSourceFromHBitmap(bmp.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
            imageTimeIcon.Source = bmpSource;

            bmp = Properties.Resources.MINES_LEFT_ICON;
            bmpSource = Imaging.CreateBitmapSourceFromHBitmap(bmp.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
            imageMinesLeftIcon.Source = bmpSource;

            OptionsData optionsData = OptionsData.Deserialize();
            if (optionsData == null)
                optionsData = new OptionsData();
            StatisticsData statisticsData = StatisticsData.Deserialize();
            if (statisticsData == null)
                statisticsData = new StatisticsData(true);
            m_gameData = new GameData (optionsData, statisticsData);
            m_gameData.CreateHexArrayFromOptions();
            mineGrid.SetGameData (m_gameData);
            mineGrid.ResizeWindowToFitHexGrid();
            mineGrid.SetOwner (this);
            UpdateMinesLeftDisplay();
            m_timeUpdateTimer = new DispatcherTimer();
            m_timeUpdateTimer.Interval = TimeSpan.FromSeconds(1);
            m_timeUpdateTimer.Tick += timer_Tick;

            F1pressed.InputGestures.Add(new KeyGesture(Key.F1));
            F2pressed.InputGestures.Add(new KeyGesture(Key.F2));
            F4pressed.InputGestures.Add(new KeyGesture(Key.F4));
            F5pressed.InputGestures.Add(new KeyGesture(Key.F5));
        }

        public void DoTimeUpdates()
        {
            if (!m_timeUpdateTimer.IsEnabled)
                m_timeUpdateTimer.Start();
            UpdateTimeDisplay();
        }

        public void StopTimeUpdates()
        {
            if (m_timeUpdateTimer.IsEnabled)
                m_timeUpdateTimer.Stop();
            UpdateTimeDisplay();
        }

        public void GameOver()
        {
            StopTimeUpdates();
            GameOverDlgWindow dlg = new GameOverDlgWindow();
            dlg.Owner = this;
            dlg.SetGameData (m_gameData);
            dlg.ShowDialog();
            if (dlg.exitWasPressed)
                Close();
            else if (dlg.playAgainWasPressed)
                DoNewGame();
        }

        private void timer_Tick (object sender, EventArgs e)
        {
            UpdateTimeDisplay();
        }

        public void UpdateTimeDisplay()
        {
            if (m_gameData.gamePlayStatus == GameData.GamePlayStatus.NOT_STARTED)
                LabelTime.Content = "---";
            else
                LabelTime.Content = m_gameData.ElapsedTime;
        }

        public void UpdateMinesLeftDisplay()
        {
            LabelMinesRemaining.Content = m_gameData.minesLeft;
        }

        public static string AppDataDir
        {
            get
            {
                return System.IO.Path.Combine (
                    Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData),
                    "Hexagonal Minesweeper 2019");
            }
        }

        private void Statistics_Click(object sender, RoutedEventArgs e)
        {
            DateTime dlgStartTime = DateTime.Now;
            StopTimeUpdates();
            StatisticsDlgWindow dlg = new StatisticsDlgWindow();
            dlg.Owner = this;
            dlg.SetStatisticsData (m_gameData.Statistics, m_gameData.Options);
            dlg.ShowDialog();
            if (m_gameData.gamePlayStatus == GameData.GamePlayStatus.RUNNING)
            {
                m_gameData.totalTimeNotCounted += DateTime.Now - dlgStartTime;
                DoTimeUpdates();
            }
        }

        private void DoNewGame()
        {
            StopTimeUpdates();
            m_gameData.CreateHexArrayFromOptions();
            UpdateMinesLeftDisplay();
            UpdateTimeDisplay();
            mineGrid.InvalidateVisual();
        }

        private void NewGame_Click(object sender, RoutedEventArgs e)
        {
            DoNewGame();
        }

        private void Options_Click(object sender, RoutedEventArgs e)
        {
            DateTime dlgStartTime = DateTime.Now;
            StopTimeUpdates();
            int oldHeight = m_gameData.Options.hexesHigh;
            int oldWidth = m_gameData.Options.hexesWide;
            int oldNumMines = m_gameData.Options.numMines;
            bool oldRightToLeft = m_gameData.Options.rightToLeft;
            OptionsData.ColorScheme oldColorScheme = m_gameData.Options.colorScheme;
            OptionsDlgWindow dlg = new OptionsDlgWindow();
            dlg.Owner = this;
            dlg.SetOptionsData(m_gameData.Options);
            if (dlg.ShowDialog() == true)
            {
                // User clicked OK, update
                if (m_gameData.Options.hexesHigh != oldHeight ||
                    m_gameData.Options.hexesWide != oldWidth ||
                    m_gameData.Options.numMines != oldNumMines ||
                    m_gameData.Options.rightToLeft != oldRightToLeft)
                {
                    m_gameData.CreateHexArrayFromOptions();
                    mineGrid.ResizeWindowToFitHexGrid();
                    mineGrid.InvalidateVisual();
                    UpdateTimeDisplay();
                    UpdateMinesLeftDisplay();
                }
                if (m_gameData.Options.colorScheme != oldColorScheme)
                {
                    mineGrid.SetBitmapsForColorScheme();
                    mineGrid.InvalidateVisual();
                }
            }
            if (m_gameData.gamePlayStatus == GameData.GamePlayStatus.RUNNING)
            {
                m_gameData.totalTimeNotCounted += DateTime.Now - dlgStartTime;
                DoTimeUpdates();
            }
        }

        private void AppExit_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        private void ViewHelp_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                System.Diagnostics.Process.Start("HexSweep2019Help.html");
            }
            catch
            {
                MessageBox.Show("Could not launch HexSweep2019Help.html");
            }
        }

        private void About_Click(object sender, RoutedEventArgs e)
        {
            DateTime dlgStartTime = DateTime.Now;
            StopTimeUpdates();
            AboutDlgWindow dlg = new AboutDlgWindow();
            dlg.Owner = this;
            dlg.ShowDialog();
            if (m_gameData.gamePlayStatus == GameData.GamePlayStatus.RUNNING)
            {
                m_gameData.totalTimeNotCounted += DateTime.Now - dlgStartTime;
                DoTimeUpdates();
            }
        }

        private DateTime m_minimizedTime = DateTime.MinValue;

        private void Window_StateChanged(object sender, EventArgs e)
        {
            switch (this.WindowState)
            {
                case WindowState.Minimized:
                    if (m_gameData.gamePlayStatus == GameData.GamePlayStatus.RUNNING)
                    {
                        StopTimeUpdates();
                        m_minimizedTime = DateTime.Now;
                    }
                    break;
                default:
                    if (m_gameData.gamePlayStatus == GameData.GamePlayStatus.RUNNING
                        &&  m_minimizedTime != DateTime.MinValue)
                    {
                        m_gameData.totalTimeNotCounted += DateTime.Now - m_minimizedTime;
                        m_minimizedTime = DateTime.MinValue;
                        DoTimeUpdates();
                    }
                    break;
            }
        }

        private int m_nextCheatStringChar = 0;
        private const string m_cheatString = "xyzzy\r\r";

        private void Window_TextInput(object sender, TextCompositionEventArgs e)
        {
            string inputChar = e.Text.ToLower();
            if (inputChar[0] == m_cheatString [m_nextCheatStringChar])
            {
                switch (m_nextCheatStringChar)
                {
                    case 5:
                        // Must have Shift held down
                        if ((Keyboard.Modifiers & ModifierKeys.Shift) != ModifierKeys.Shift)
                            return;
                        break;
                    case 6:
                        // Must not have Shift held down
                        if ((Keyboard.Modifiers & ModifierKeys.Shift) == ModifierKeys.Shift)
                            return;
                        break;
                }
                ++m_nextCheatStringChar;
                if (m_nextCheatStringChar == m_cheatString.Length)
                {
                    // Complete sequence entered; toggle cheat mode
                    mineGrid.cheating = !mineGrid.cheating;
                    m_nextCheatStringChar = 0;  // return to the beginning to await next toggle
                }
            }
            else if (m_nextCheatStringChar > 0)
                m_nextCheatStringChar = 0;  // c-c-c-c-combo breaker!
        }

        private void F1KeyPressed (object sender, ExecutedRoutedEventArgs e)
        {
            ViewHelp_Click (sender, e);
        }

        private void F2KeyPressed(object sender, ExecutedRoutedEventArgs e)
        {
            DoNewGame();
        }

        private void F4KeyPressed(object sender, ExecutedRoutedEventArgs e)
        {
            Statistics_Click (sender, e);
        }

        private void F5KeyPressed(object sender, ExecutedRoutedEventArgs e)
        {
            Options_Click (sender, e);
        }
    }
}
