﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Interop;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Hexagonal_Minesweeper_2019
{
    /// <summary>
    /// Interaction logic for MineGrid.xaml
    /// </summary>
    public partial class MineGrid : UserControl
    {
        private MainWindow m_owner = null;
        private GameData.Hex m_mouseDownHex = null;
        private GameData m_gameData;
        private BitmapSource m_coveredBitmap, m_uncoveredBitmap, m_uncoveredCheatingMineBitmap, m_markedOverlay;
        private BitmapSource m_mineOverlay, m_mineOverlayFaded, m_explodedMineBitmap, m_questionMarkOverlay;
        private BitmapSource [] m_numberBitmap;

        public bool cheating = false;

        public MineGrid()
        {
            InitializeComponent();

            System.Drawing.Bitmap bmp = Properties.Resources.MINE_OVERLAY;
            m_mineOverlay = Imaging.CreateBitmapSourceFromHBitmap(bmp.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
            bmp = Properties.Resources.MINE_OVERLAY_FADED;
            m_mineOverlayFaded = Imaging.CreateBitmapSourceFromHBitmap(bmp.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
            bmp = Properties.Resources.BOOM;
            m_explodedMineBitmap = Imaging.CreateBitmapSourceFromHBitmap(bmp.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
            bmp = Properties.Resources.UNCOVERED_CHEATING_MINE;
            m_uncoveredCheatingMineBitmap = Imaging.CreateBitmapSourceFromHBitmap(bmp.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());

            m_numberBitmap = new BitmapSource[7];
            m_numberBitmap[0] = null;
            bmp = Properties.Resources.NUMBER_1;
            m_numberBitmap[1] = Imaging.CreateBitmapSourceFromHBitmap(bmp.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
            bmp = Properties.Resources.NUMBER_2;
            m_numberBitmap[2] = Imaging.CreateBitmapSourceFromHBitmap(bmp.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
            bmp = Properties.Resources.NUMBER_3;
            m_numberBitmap[3] = Imaging.CreateBitmapSourceFromHBitmap(bmp.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
            bmp = Properties.Resources.NUMBER_4;
            m_numberBitmap[4] = Imaging.CreateBitmapSourceFromHBitmap(bmp.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
            bmp = Properties.Resources.NUMBER_5;
            m_numberBitmap[5] = Imaging.CreateBitmapSourceFromHBitmap(bmp.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
            bmp = Properties.Resources.NUMBER_6;
            m_numberBitmap[6] = Imaging.CreateBitmapSourceFromHBitmap(bmp.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
        }

        public void SetBitmapsForColorScheme()
        {
            System.Drawing.Bitmap coveredBmp, uncoveredBmp, markedBmp, questionBmp;
            switch (m_gameData.Options.colorScheme)
            {
                case OptionsData.ColorScheme.BLUE:
                    coveredBmp = Properties.Resources.COVERED_blue;
                    uncoveredBmp = Properties.Resources.UNCOVERED_blue;
                    markedBmp = Properties.Resources.MARKED_OVERLAY_darkBkgd;
                    questionBmp = Properties.Resources.QUESTION_MARK_OVERLAY_darkBkgd;
                    break;
                case OptionsData.ColorScheme.CLASSIC:
                default:
                    coveredBmp = Properties.Resources.COVERED;
                    uncoveredBmp = Properties.Resources.UNCOVERED;
                    markedBmp = Properties.Resources.MARKED_OVERLAY_lightBkgd;
                    questionBmp = Properties.Resources.QUESTION_MARK_OVERLAY_lightBkgd;
                    break;
            }
            m_coveredBitmap = Imaging.CreateBitmapSourceFromHBitmap(coveredBmp.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
            m_uncoveredBitmap = Imaging.CreateBitmapSourceFromHBitmap(uncoveredBmp.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
            m_markedOverlay = Imaging.CreateBitmapSourceFromHBitmap(markedBmp.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
            m_questionMarkOverlay = Imaging.CreateBitmapSourceFromHBitmap(questionBmp.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
        }

        public void SetOwner (MainWindow win)
        {
            m_owner = win;
        }

        public void SetGameData (GameData gd)
        {
            m_gameData = gd;
            SetBitmapsForColorScheme();
        }

        public void ResizeWindowToFitHexGrid()
        {
            Thickness margin = Margin;
            int newWidth = m_gameData.HexesWide * GameData.HEX_COL_WIDTH +
                GameData.HEX_ROW_HEIGHT - GameData.HEX_COL_WIDTH + 2;
            int newHeight = m_gameData.HexesHigh * GameData.HEX_ROW_HEIGHT +
                GameData.HEX_ROW_HEIGHT / 2 + 2;
            Grid parent = Parent as Grid;
            if (parent != null)
            {
                parent.Width = margin.Left + newWidth + margin.Right;
                parent.Height = margin.Top + newHeight + margin.Bottom;
            }
        }

        private void DrawHex (DrawingContext dc, GameData.Hex hex)
        {
            int x, y;
            hex.GetGridCoords (out x, out y);
            Rect r = new Rect (x, y, 17, 17);

            if (m_gameData.gamePlayStatus == GameData.GamePlayStatus.EXPLODED  &&  hex.hasMine)
            {
                if (hex == m_gameData.deathHex)
                    dc.DrawImage (m_explodedMineBitmap, r);
                else
                {
                    dc.DrawImage (m_uncoveredBitmap, r);
                    if (hex.covered == GameData.HexStatus.MARKED)
                    {
                        dc.DrawImage(m_mineOverlayFaded, r);
                        dc.DrawImage(m_markedOverlay, r);
                    }
                    else
                    {
                        dc.DrawImage(m_mineOverlay, r);
                        if (hex.covered == GameData.HexStatus.QUESTION_MARKED)
                            dc.DrawImage(m_questionMarkOverlay, r);
                    }
                }
            }
            else switch (hex.covered)
            {
                case GameData.HexStatus.COVERED:
                {
                    List<GameData.Hex> coveredHexesToBeDrawnAsUncovered = new List<GameData.Hex>();
                    if (m_mouseDownHex != null)
                    {
                        coveredHexesToBeDrawnAsUncovered.Add(m_mouseDownHex);
                        if (Mouse.RightButton == MouseButtonState.Pressed)
                        {
                            List<GameData.Hex> extraHexes = m_gameData.NeighboringCoveredUnmarkedHexes(m_mouseDownHex);
                            coveredHexesToBeDrawnAsUncovered.AddRange(extraHexes);
                        }
                    }
                    if (coveredHexesToBeDrawnAsUncovered.Contains(hex))
                    {
                        if (cheating  &&  hex.hasMine)
                           dc.DrawImage(m_uncoveredCheatingMineBitmap, r);
                        else
                           dc.DrawImage(m_uncoveredBitmap, r);
                    }
                    else
                        dc.DrawImage(m_coveredBitmap, r);
                    break;
                }
                case GameData.HexStatus.UNCOVERED:
                {
                    dc.DrawImage (m_uncoveredBitmap, r);
                    int numMinedNeighbors = m_gameData.CountNeighborsWithMines(hex);
                    if (numMinedNeighbors > 0)
                        dc.DrawImage (m_numberBitmap[numMinedNeighbors], r);
                    break;
                }
                case GameData.HexStatus.MARKED:
                    if (m_gameData.gamePlayStatus == GameData.GamePlayStatus.EXPLODED  &&  !hex.hasMine)
                    {
                        // Draw a pale imitation of a mine, with a 2-pixel-thick red X over it
                        dc.DrawImage(m_uncoveredBitmap, r);
                        dc.DrawImage(m_mineOverlayFaded, r);
                        Pen pen = new Pen(Brushes.DarkRed, 2.0);
                        dc.DrawLine(pen, new Point(x + 3, y + 3), new Point(x + 14, y + 14));
                        dc.DrawLine(pen, new Point(x + 3, y + 14), new Point(x + 14, y + 3));
                    }
                    else
                    {
                        dc.DrawImage(m_coveredBitmap, r);
                        dc.DrawImage(m_markedOverlay, r);
                    }
                    break;
                case GameData.HexStatus.QUESTION_MARKED:
                    dc.DrawImage (m_coveredBitmap, r);
                    dc.DrawImage (m_questionMarkOverlay, r);
                    break;
            }
        }

        protected override void OnRender (DrawingContext dc)
        {
            base.OnRender(dc);
            Rect boundsRect = new Rect(0, 0, ActualWidth, ActualHeight);
            dc.DrawRectangle(Brushes.LightGray, null, boundsRect);

            if (m_gameData != null)
                for (int col = 0; col < m_gameData.HexesWide; ++col)
                    for (int row = 0; row < m_gameData.HexesHigh; ++row)
                    {
                        GameData.Hex hex = m_gameData.GetHex(col, row);
                        DrawHex(dc, hex);
                    }
        }

        private GameData.Hex GetHexFromMouseEvent (MouseEventArgs e)
        {
            Point p = e.GetPosition(this);
            // Adjust for margins
            p.X -= 2;
            GameData.Hex hex = m_gameData.FindHex((int)p.X, (int)p.Y);
            return hex;
        }

        private void SetMouseDownHex (MouseEventArgs e)
        {
            GameData.Hex hex = GetHexFromMouseEvent (e);
            if (hex != m_mouseDownHex)
            {
                m_mouseDownHex = hex;
                InvalidateVisual();
            }
        }

        // Returns true if the game ends
        private bool DoClearAroundMouseDownHex()
        {
            if (m_mouseDownHex == null)
                return false;
            if (m_mouseDownHex.covered != GameData.HexStatus.UNCOVERED)
                return false;
            m_gameData.ClearAroundHex(m_mouseDownHex);
            return  m_gameData.gamePlayStatus == GameData.GamePlayStatus.WON  ||
                    m_gameData.gamePlayStatus == GameData.GamePlayStatus.EXPLODED;
        }

        private void UserControl_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (m_gameData.gamePlayStatus == GameData.GamePlayStatus.WON ||
                m_gameData.gamePlayStatus == GameData.GamePlayStatus.EXPLODED)
                    return; // allow no interaction if the game is over.
            CaptureMouse();
            SetMouseDownHex (e);
            if (e.RightButton == MouseButtonState.Pressed)
            {
                // We're preparing to uncover ALL unmarked covered hexes around
                // m_mouseDownHex, instead of merely uncovering m_mouseDownHex.
                InvalidateVisual();
            }
        }

        private void UserControl_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (IsMouseCaptured)
            {
                bool gameOver = false;
                ReleaseMouseCapture();
                SetMouseDownHex(e);
                if (m_mouseDownHex != null)
                    if (e.RightButton == MouseButtonState.Pressed)
                        gameOver = DoClearAroundMouseDownHex();
                    else
                    {
                        if (m_mouseDownHex.covered == GameData.HexStatus.QUESTION_MARKED)
                            m_mouseDownHex.covered = GameData.HexStatus.COVERED;
                        m_gameData.UncoverHex (m_mouseDownHex);
                        if (m_gameData.gamePlayStatus == GameData.GamePlayStatus.RUNNING)
                            m_owner.DoTimeUpdates();
                    }
                m_mouseDownHex = null;
                InvalidateVisual();
                if (m_gameData.gamePlayStatus == GameData.GamePlayStatus.WON ||
                    m_gameData.gamePlayStatus == GameData.GamePlayStatus.EXPLODED)
                        m_owner.GameOver();
            }
        }

        private void UserControl_MouseMove(object sender, MouseEventArgs e)
        {
            if (IsMouseCaptured)
                SetMouseDownHex(e);
        }

        private void UserControl_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (m_gameData.gamePlayStatus == GameData.GamePlayStatus.WON ||
                m_gameData.gamePlayStatus == GameData.GamePlayStatus.EXPLODED)
                    return; // allow no interaction if the game is over.
            if (IsMouseCaptured)
            {
                // now we're preparing to uncover ALL unmarked covered hexes around
                // m_mouseDownHex, instead of merely uncovering m_mouseDownHex.
                InvalidateVisual();
            }
            else
            {
                GameData.Hex hex = GetHexFromMouseEvent (e);
                if (hex != null)
                    CycleMark(hex);
            }
        }

        private void UserControl_MouseRightButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (IsMouseCaptured)
            {
                // The mouse is only captured if the LEFT mouse button is down,
                // and the game isn't over.
                ReleaseMouseCapture();
                SetMouseDownHex(e);
                bool gameOver = DoClearAroundMouseDownHex();
                m_mouseDownHex = null;
                InvalidateVisual();
                if (gameOver)
                    m_owner.GameOver();
            }
        }

        private void CycleMark (GameData.Hex hex)
        {
            if (hex.covered == GameData.HexStatus.COVERED)
            {
                hex.covered = GameData.HexStatus.MARKED;
                --m_gameData.minesLeft;
                m_owner.UpdateMinesLeftDisplay();
            }
            else if (hex.covered == GameData.HexStatus.QUESTION_MARKED)
                hex.covered = GameData.HexStatus.COVERED;
            else if (hex.covered == GameData.HexStatus.MARKED)
            {
                if (m_gameData.Options.useQuestionMarks)
                    hex.covered = GameData.HexStatus.QUESTION_MARKED;
                else
                    hex.covered = GameData.HexStatus.COVERED;
                ++m_gameData.minesLeft;
                m_owner.UpdateMinesLeftDisplay();
            }
            InvalidateVisual();
        }
    }
}
