﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Serialization;

namespace Hexagonal_Minesweeper_2019
{
    public class OptionsData
    {
        public const int MIN_HEXES_WIDE = 12;
        public const int MIN_HEXES_HIGH = 8;
        public const int MAX_HEXES_WIDE = 48;
        public const int MAX_HEXES_HIGH = 24;

        public int hexesWide = MIN_HEXES_WIDE;
        public int hexesHigh = MIN_HEXES_HIGH;
        public int numMines = 15;
        public bool useQuestionMarks = true;

        // If rightToLeft is true, mines MUST be uncovered starting at the rightmost
        // edge and working your way leftward.  The game is won the moment a hex on the
        // leftmost row is uncovered.
        public bool rightToLeft = false;

        public enum ColorScheme
        {
            CLASSIC,
            BLUE
        };
        public ColorScheme colorScheme = ColorScheme.CLASSIC;

        public enum Difficulty
        {
            BEGINNER,
            INTERMEDIATE,
            EXPERT,
            CUSTOM
        }

        public static Difficulty GetDifficultyFromValues (int mines, int columns, int rows)
        {
            if (columns == MIN_HEXES_WIDE && rows == MIN_HEXES_HIGH && mines == 15)
                return Difficulty.BEGINNER;
            else if (columns == 24 && rows == 16 && mines == 60)
                return Difficulty.INTERMEDIATE;
            else if (columns == 36 && rows == 16 && mines == 135)
                return Difficulty.EXPERT;
            else
                return Difficulty.CUSTOM;
        }

        public Difficulty GetDifficulty()
        {
            return GetDifficultyFromValues (numMines, hexesWide, hexesHigh);
        }

        public bool Serialize()
        {
            FileStream stream = null;
            try
            {
                string dir = MainWindow.AppDataDir;
                if (!Directory.Exists(dir))
                    Directory.CreateDirectory(dir);
                string filename = Path.Combine(dir, "options.xml");
                stream = new FileStream(filename, FileMode.Create, FileAccess.Write, FileShare.None);
                XmlSerializer writer = new XmlSerializer(typeof(OptionsData));
                writer.Serialize(stream, this);
                stream.Close();
                return true;
            }
            catch
            {
                if (stream != null)
                    stream.Close();
                return false;
            }
        }

        public static OptionsData Deserialize()
        {
            try
            {
                string dir = MainWindow.AppDataDir;
                if (!Directory.Exists(dir))
                    return null;

                string filename = Path.Combine(dir, "options.xml");
                FileStream stream = new FileStream(filename, FileMode.Open, FileAccess.Read, FileShare.None);
                XmlSerializer reader = new XmlSerializer(typeof(OptionsData));
                OptionsData data = reader.Deserialize (stream) as OptionsData;
                stream.Close();
                return data;
            }
            catch
            {
                return null;
            }
        }
    }
}
