﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace Hexagonal_Minesweeper_2019
{
    /// <summary>
    /// Interaction logic for OptionsDlgWindow.xaml
    /// </summary>
    public partial class OptionsDlgWindow : Window
    {
        private OptionsData m_optionsData;

        public OptionsDlgWindow()
        {
            InitializeComponent();

            labelColumns.Content = "Columns wide (" + OptionsData.MIN_HEXES_WIDE
                + "-" + OptionsData.MAX_HEXES_WIDE + "):";
            labelRows.Content = "Rows high (" + OptionsData.MIN_HEXES_HIGH
                + "-" + OptionsData.MAX_HEXES_HIGH + "):";
        }

        public void SetOptionsData (OptionsData optionsData)
        {
            m_optionsData = optionsData;

            OptionsData.Difficulty diff = m_optionsData.GetDifficulty();
            switch (diff)
            {
                case OptionsData.Difficulty.BEGINNER:
                    radioButtonBeginner.IsChecked = true;
                    break;
                case OptionsData.Difficulty.INTERMEDIATE:
                    radioButtonIntermediate.IsChecked = true;
                    break;
                case OptionsData.Difficulty.EXPERT:
                    radioButtonExpert.IsChecked = true;
                    break;
                case OptionsData.Difficulty.CUSTOM:
                    radioButtonCustom.IsChecked = true;
                    break;
            }
            EnableAndDisableControls();
            textBoxMines.Text = m_optionsData.numMines.ToString();
            textBoxColumns.Text = m_optionsData.hexesWide.ToString();
            textBoxRows.Text = m_optionsData.hexesHigh.ToString();

            checkBoxQuestionMarks.IsChecked = m_optionsData.useQuestionMarks;
            checkBoxRightToLeft.IsChecked = m_optionsData.rightToLeft;
            checkBoxBlue.IsChecked = m_optionsData.colorScheme == OptionsData.ColorScheme.BLUE;
        }

        private void EnableAndDisableControls()
        {
            bool enable = radioButtonCustom.IsChecked == true;
            labelMines.IsEnabled = enable;
            labelColumns.IsEnabled = enable;
            labelRows.IsEnabled = enable;
            textBoxMines.IsEnabled = enable;
            textBoxColumns.IsEnabled = enable;
            textBoxRows.IsEnabled = enable;
        }

        private void buttonCancel_Click(object sender, RoutedEventArgs e)
        {

        }

        private void radioButtonBeginner_Checked(object sender, RoutedEventArgs e)
        {
            textBoxMines.Text = 15.ToString();
            textBoxColumns.Text = OptionsData.MIN_HEXES_WIDE.ToString();
            textBoxRows.Text = OptionsData.MIN_HEXES_HIGH.ToString();
            EnableAndDisableControls();
        }

        private void radioButtonIntermediate_Checked(object sender, RoutedEventArgs e)
        {
            textBoxMines.Text = 60.ToString();
            textBoxColumns.Text = 24.ToString();
            textBoxRows.Text = 16.ToString();
            EnableAndDisableControls();
        }

        private void radioButtonExpert_Checked(object sender, RoutedEventArgs e)
        {
            textBoxMines.Text = 135.ToString();
            textBoxColumns.Text = 36.ToString();
            textBoxRows.Text = 16.ToString();
            EnableAndDisableControls();
        }

        private void radioButtonCustom_Checked(object sender, RoutedEventArgs e)
        {
            EnableAndDisableControls();
        }

        private void buttonOK_Click(object sender, RoutedEventArgs e)
        {
            // Validate height, width, and number of mines
            int mines, width, height;

            if (!int.TryParse (textBoxColumns.Text, out width)  ||
                width < OptionsData.MIN_HEXES_WIDE  ||
                width > OptionsData.MAX_HEXES_WIDE)
            {
                MessageBox.Show("Columns wide must be between " + OptionsData.MIN_HEXES_WIDE
                    + " and " + OptionsData.MAX_HEXES_WIDE + ".");
                return;
            }

            if (!int.TryParse(textBoxRows.Text, out height) ||
                height < OptionsData.MIN_HEXES_HIGH ||
                height > OptionsData.MAX_HEXES_HIGH)
            {
                MessageBox.Show("Rows high must be between " + OptionsData.MIN_HEXES_HIGH
                    + " and " + OptionsData.MAX_HEXES_HIGH + ".");
                return;
            }

            int maxMines = width * height * 6 / 7;
            if (!int.TryParse(textBoxMines.Text, out mines) ||
                mines < 10 || mines > maxMines)
            {
                string msg = "For a " + width + " by " + height +
                    "grid, mines must be between 10 and " + maxMines + ".";
                MessageBox.Show(msg);
                return;
            }

            m_optionsData.hexesHigh = height;
            m_optionsData.hexesWide = width;
            m_optionsData.numMines = mines;
            m_optionsData.useQuestionMarks = checkBoxQuestionMarks.IsChecked == true;
            m_optionsData.rightToLeft = checkBoxRightToLeft.IsChecked == true;
            m_optionsData.colorScheme = checkBoxBlue.IsChecked == true ?
                OptionsData.ColorScheme.BLUE : OptionsData.ColorScheme.CLASSIC;
            m_optionsData.Serialize();
            DialogResult = true;
            Close();
        }
    }
}
