﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace Hexagonal_Minesweeper_2019
{
    /// <summary>
    /// Interaction logic for StatisticsDlgWindow.xaml
    /// </summary>
    public partial class StatisticsDlgWindow : Window
    {
        private StatisticsData m_statisticsData;

        public StatisticsDlgWindow()
        {
            InitializeComponent();
        }

        public void SetStatisticsData (StatisticsData statisticsData, OptionsData opts)
        {
            m_statisticsData = statisticsData;
            StatisticsData.StatisicsForOneDifficulty initialSelection = m_statisticsData.FindByDifficulty
                (opts.numMines, opts.hexesWide, opts.hexesHigh, opts.rightToLeft);  // may be null
            foreach (StatisticsData.StatisicsForOneDifficulty sfod in m_statisticsData.statCollection)
            {
                int newIndex = listBoxDifficulties.Items.Add (sfod.DifficultyName);
                if (sfod == initialSelection)
                {
                    listBoxDifficulties.SelectedIndex = newIndex;
                    Populate(sfod);
                }
            }
        }

        private void Populate (StatisticsData.StatisicsForOneDifficulty sfod)
        {
            for (int i = 0; i < StatisticsData.StatisicsForOneDifficulty.NUM_BEST_TIMES; ++i)
            {
                Label bestTimeLabel = null;
                switch (i)
                {
                    case 0:
                        bestTimeLabel = labelBestTime1;
                        break;
                    case 1:
                        bestTimeLabel = labelBestTime2;
                        break;
                    case 2:
                        bestTimeLabel = labelBestTime3;
                        break;
                    case 3:
                        bestTimeLabel = labelBestTime4;
                        break;
                    case 4:
                        bestTimeLabel = labelBestTime5;
                        break;
                }
                if (bestTimeLabel == null)
                    continue;
                if (sfod.bestTimes[i] >= 999)
                {
                    bestTimeLabel.Content = "";
                    continue;
                }
                bestTimeLabel.Content = sfod.bestTimes[i] + "\t" + sfod.dateOfBestTimes[i].ToShortDateString();
            }
            labelGamesPlayed.Content = sfod.gamesPlayed;
            labelGamesWon.Content = sfod.gamesWon;
            labelWinPercentage.Content = sfod.WinPercentage + "%";
            labelLongestWinningStreak.Content = sfod.longestWinStreak;
            labelLongestLosingStreak.Content = sfod.longestLoseStreak;
            labelCurrentStreak.Content = sfod.currentStreak;
        }

        private void buttonReset_Click(object sender, RoutedEventArgs e)
        {
            if (MessageBox.Show ("Do you want to reset all your statistics to zero?", "Reset statistics", MessageBoxButton.YesNo)
                != MessageBoxResult.Yes)
                    return;
            m_statisticsData.ResetAll();
            StatisticsData.StatisicsForOneDifficulty sfod = m_statisticsData.statCollection[listBoxDifficulties.SelectedIndex];
            Populate(sfod);
        }

        private void listBoxDifficulties_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            StatisticsData.StatisicsForOneDifficulty sfod = m_statisticsData.statCollection[listBoxDifficulties.SelectedIndex];
            Populate(sfod);
        }
    }
}
